# ============================================================
#
# Copyright (C) 2012, 2013 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
# Copyright (C) 2013 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
# Copyright (C) 2014 Norman Koester <nkoester at techfak dot uni-bielefeld dot de>
#
# This file may be licensed under the terms of the
# GNU Lesser General Public License Version 3 (the ``LGPL''),
# or (at your option) any later version.
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the LGPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the LGPL along with this
# program. If not, go to http://www.gnu.org/licenses/lgpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# The development of this software was supported by:
#   CoR-Lab, Research Institute for Cognition and Robotics
#     Bielefeld University
#
# ============================================================

# mark-start::opencv
import cv2
import numpy
# mark-end::opencv

# mark-start::exit-strategy
import signal
doRun = True
def signal_handler(signal, frame):
    global doRun
    print "exiting."
    doRun = False
signal.signal(signal.SIGINT, signal_handler)
# mark-end::exit-strategy

# mark-start::draw-keypoints
def drawKeyPoints(target, mask):
    target[mask]=[255,0,0]
# mark-end::draw-keypoints

# mark-start::commandline-arguments
import argparse

def parseArguments():

    parser = argparse.ArgumentParser(description='An example program sending some magic numbers')
    parser.add_argument('--inscope', type=str, default='/video',
                        help='scope to receive images from')
    parser.add_argument('--outscope', type=str, default='/results',
                        help='scope to send processing results to')
    return parser.parse_args()

def detectHarris():

    args = parseArguments()
    # mark-end::commandline-arguments

    # mark-start::import-rsb
    import rsb
    # mark-end::import-rsb
    # mark-start::repository-function
    from rsb.converter import registerGlobalConverter
    # mark-end::repository-function
    # mark-start::image-converter
    from rstconverters.opencv import IplimageConverter
    # mark-end::image-converter
    # mark-start::import-queue
    from Queue import Queue
    # mark-end::import-queue
    # mark-start::rst-imports
    import rstsandbox
    from rst.geometry.PointCloud2DInt_pb2 import PointCloud2DInt
    from rsb.converter import ProtocolBufferConverter
    # mark-end::rst-imports

    # mark-start::converter-registration
    registerGlobalConverter(IplimageConverter())
    rsb.setDefaultParticipantConfig(rsb.ParticipantConfig.fromDefaultSources())
    # mark-end::converter-registration

    # mark-start::rst-registration
    registerGlobalConverter(ProtocolBufferConverter(messageClass=PointCloud2DInt))
    rsb.setDefaultParticipantConfig(rsb.ParticipantConfig.fromDefaultSources())
    # mark-end::rst-registration

    # mark-start::listener
    listener = rsb.createListener(args.inscope)
    # mark-end::listener

    # mark-start::queue
    lastImage = Queue(1)
    # mark-end::queue
    # mark-start::handler
    def addLastImage(imageEvent):
        try:
            lastImage.get(False)
        except:
            pass
        lastImage.put(numpy.asarray(imageEvent.data[:,:]), False)
    listener.addHandler(addLastImage)
    # mark-end::handler

    # mark-start::informer
    informer = rsb.createInformer(args.outscope, dataType=PointCloud2DInt)
    # mark-end::informer

    # mark-start::working-loop
    while doRun:

        image = lastImage.get(True)

        cv2.imshow("input", image)
        cv2.waitKey(1)
        # mark-end::working-loop

        # mark-start::cornerharris
        imageGray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        cornersResult = cv2.cornerHarris(imageGray,2,3,0.04)
        cornersResult = cv2.dilate(cornersResult,None)
        mask = cornersResult>0.01*cornersResult.max()
        # mark-end::cornerharris

        # mark-start::display-keypoints
        imageWithKeyPoints = numpy.array(image)
        drawKeyPoints(imageWithKeyPoints, mask)

        cv2.imshow("result", imageWithKeyPoints)
        cv2.waitKey(1)
        # mark-end::display-keypoints

        # mark-start::publish
        cloud = PointCloud2DInt()

        x,y = numpy.where(mask)
        for xe, ye in zip(x,y):
            p = cloud.points.add()
            p.x = int(xe)
            p.y = int(ye)

        informer.publishData(cloud)
        # mark-end::publish

    listener.deactivate()
    informer.deactivate()
